# utils package
# This package contains utility functions for the project

# Import functions from submodules to make them available directly from the package
from .string_utils import capitalize_words, reverse_string
from .math_utils import add, multiply