# models package
# This package contains data models for the project

# Import classes from submodules to make them available directly from the package
from .user import User
from .product import Product