"""
HLCalculator - An advanced command line calculator
"""
from .calculator import ScientificCalculator
from .parser import parse_arguments

__version__ = "0.1.0"
__all__ = ['ScientificCalculator', 'parse_arguments']
